@@ -11,55 +11,57 @@ Tornado-based server "on the side".
 The framework being used must support web sockets.
 """
 
+import argparse
 import io
+import json
 import mimetypes
+from pathlib import Path
+import signal
+import socket
 
 try:
     import tornado
-except ImportError:
-    raise RuntimeError("This example requires tornado.")
+except ImportError as err:
+    raise RuntimeError("This example requires tornado.") from err
 import tornado.web
 import tornado.httpserver
 import tornado.ioloop
 import tornado.websocket
 
 
-from matplotlib.backends.backend_webagg_core import (
+import matplotlib as mpl
+from matplotlib.backends.backend_webagg import (
     FigureManagerWebAgg, new_figure_manager_given_figure)
 from matplotlib.figure import Figure
 
 import numpy as np
 
-import json
-
 
 def create_figure():
     """
     Creates a simple example figure.
     """
     fig = Figure()
-    a = fig.add_subplot(111)
+    ax = fig.add_subplot()
     t = np.arange(0.0, 3.0, 0.01)
     s = np.sin(2 * np.pi * t)
-    a.plot(t, s)
+    ax.plot(t, s)
     return fig
 
 
 # The following is the content of the web page.  You would normally
 # generate this using some sort of template facility in your web
 # framework, but here we just use Python string formatting.
-html_content = """
-<html>
+html_content = """<!DOCTYPE html>
+<html lang="en">
   <head>
     <!-- TODO: There should be a way to include all of the required javascript
                and CSS so matplotlib can add to the set in the future if it
                needs to. -->
     <link rel="stylesheet" href="_static/css/page.css" type="text/css">
-    <link rel="stylesheet" href="_static/css/boilerplate.css" type="text/css" />
-    <link rel="stylesheet" href="_static/css/fbm.css" type="text/css" />
-    <link rel="stylesheet" href="_static/jquery-ui-1.12.1/jquery-ui.min.css" >
-    <script src="_static/jquery-ui-1.12.1/external/jquery/jquery.js"></script>
-    <script src="_static/jquery-ui-1.12.1/jquery-ui.min.js"></script>
+    <link rel="stylesheet" href="_static/css/boilerplate.css" type="text/css">
+    <link rel="stylesheet" href="_static/css/fbm.css" type="text/css">
+    <link rel="stylesheet" href="_static/css/mpl.css" type="text/css">
     <script src="mpl.js"></script>
 
     <script>
@@ -70,7 +72,15 @@ html_content = """
         window.open('download.' + format, '_blank');
       };
 
-      $(document).ready(
+      function ready(fn) {
+        if (document.readyState != "loading") {
+          fn();
+        } else {
+          document.addEventListener("DOMContentLoaded", fn);
+        }
+      }
+
+      ready(
         function() {
           /* It is up to the application to provide a websocket that the figure
              will use to communicate to the server.  This websocket object can
@@ -88,7 +98,7 @@ html_content = """
               // A function called when a file type is selected for download
               ondownload,
               // The HTML element in which to place the figure
-              $('div#figure'));
+              document.getElementById("figure"));
         }
       );
     </script>
@@ -112,7 +122,7 @@ class MyApplication(tornado.web.Application):
 
         def get(self):
             manager = self.application.manager
-            ws_uri = "ws://{req.host}/".format(req=self.request)
+            ws_uri = f"ws://{self.request.host}/"
             content = html_content % {
                 "ws_uri": ws_uri, "fig_id": manager.num}
             self.write(content)
@@ -196,8 +206,8 @@ class MyApplication(tornado.web.Application):
             if self.supports_binary:
                 self.write_message(blob, binary=True)
             else:
-                data_uri = "data:image/png;base64,{0}".format(
-                    blob.encode('base64').replace('\n', ''))
+                data_uri = ("data:image/png;base64," +
+                            blob.encode('base64').replace('\n', ''))
                 self.write_message(data_uri)
 
     def __init__(self, figure):
@@ -210,6 +220,11 @@ class MyApplication(tornado.web.Application):
              tornado.web.StaticFileHandler,
              {'path': FigureManagerWebAgg.get_static_file_path()}),
 
+            # Static images for the toolbar
+            (r'/_images/(.*)',
+             tornado.web.StaticFileHandler,
+             {'path': Path(mpl.get_data_path(), 'images')}),
+
             # The page that contains all of the pieces
             ('/', self.MainPage),
 
@@ -225,13 +240,36 @@ class MyApplication(tornado.web.Application):
 
 
 if __name__ == "__main__":
+    parser = argparse.ArgumentParser()
+    parser.add_argument('-p', '--port', type=int, default=8080,
+                        help='Port to listen on (0 for a random port).')
+    args = parser.parse_args()
+
     figure = create_figure()
     application = MyApplication(figure)
 
     http_server = tornado.httpserver.HTTPServer(application)
-    http_server.listen(8080)
+    sockets = tornado.netutil.bind_sockets(args.port, '')
+    http_server.add_sockets(sockets)
 
-    print("http://127.0.0.1:8080/")
+    for s in sockets:
+        addr, port = s.getsockname()[:2]
+        if s.family is socket.AF_INET6:
+            addr = f'[{addr}]'
+        print(f"Listening on http://{addr}:{port}/")
     print("Press Ctrl+C to quit")
 
-    tornado.ioloop.IOLoop.instance().start()
+    ioloop = tornado.ioloop.IOLoop.instance()
+
+    def shutdown():
+        ioloop.stop()
+        print("Server stopped")
+
+    old_handler = signal.signal(
+        signal.SIGINT,
+        lambda sig, frame: ioloop.add_callback_from_signal(shutdown))
+
+    try:
+        ioloop.start()
+    finally:
+        signal.signal(signal.SIGINT, old_handler)
